'use strict';

var _expect = require('expect');

var _createFieldArrayProps = require('../createFieldArrayProps');

var _createFieldArrayProps2 = _interopRequireDefault(_createFieldArrayProps);

var _plain = require('../structure/plain');

var _plain2 = _interopRequireDefault(_plain);

var _expectations = require('../structure/plain/expectations');

var _expectations2 = _interopRequireDefault(_expectations);

var _immutable = require('../structure/immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _expectations3 = require('../structure/immutable/expectations');

var _expectations4 = _interopRequireDefault(_expectations3);

var _addExpectations = require('./addExpectations');

var _addExpectations2 = _interopRequireDefault(_addExpectations);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var describeCreateFieldProps = function describeCreateFieldProps(name, structure, expect) {
  var fromJS = structure.fromJS;
  var getIn = structure.getIn;
  var deepEqual = structure.deepEqual;
  var size = structure.size;

  var defaultProps = [deepEqual, getIn, size, 'foo'];

  describe(name, function () {
    it('should pass props through', function () {
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{ otherProp: 'hello' }])).otherProp).toBe('hello');
    });

    it('should calculate dirty/pristine', function () {
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        initial: fromJS(['bar']),
        value: fromJS(['bar'])
      }])).dirty).toBe(false);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        initial: fromJS(['bar']),
        value: fromJS(['bar'])
      }])).pristine).toBe(true);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        initial: fromJS(['bar']),
        value: fromJS(['baz'])
      }])).dirty).toBe(true);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        initial: fromJS(['bar']),
        value: fromJS(['baz'])
      }])).pristine).toBe(false);
    });

    it('should provide length', function () {
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS([])
      }])).length).toBe(0);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a'])
      }])).length).toBe(1);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b'])
      }])).length).toBe(2);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b', 'c'])
      }])).length).toBe(3);
    });

    it('should provide errors', function () {
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{}, 'Sync Error'])).error).toBe('Sync Error');
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{}, 'Sync Error'])).valid).toBe(false);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{}, 'Sync Error'])).invalid).toBe(true);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        asyncError: 'Async Error'
      }])).error).toBe('Async Error');
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        asyncError: 'Async Error'
      }])).valid).toBe(false);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        asyncError: 'Async Error'
      }])).invalid).toBe(true);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        submitError: 'Submit Error'
      }])).error).toBe('Submit Error');
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        submitError: 'Submit Error'
      }])).valid).toBe(false);
      expect(_createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        submitError: 'Submit Error'
      }])).invalid).toBe(true);
    });

    it('should provide push', function () {
      var arrayPush = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b']),
        arrayPush: arrayPush
      }]));
      expect(result.push).toBeA('function');
      expect(arrayPush).toNotHaveBeenCalled();
      expect(result.push('c')).toNotExist();
      expect(arrayPush).toHaveBeenCalled().toHaveBeenCalledWith('c');
    });

    it('should provide pop', function () {
      var arrayPop = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b', 'c']),
        arrayPop: arrayPop
      }]));
      expect(result.pop).toBeA('function');
      expect(arrayPop).toNotHaveBeenCalled();
      expect(result.pop()).toBe('c');
      expect(arrayPop).toHaveBeenCalled().toHaveBeenCalledWith();
    });

    it('should provide insert', function () {
      var arrayInsert = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b']),
        arrayInsert: arrayInsert
      }]));
      expect(result.insert).toBeA('function');
      expect(arrayInsert).toNotHaveBeenCalled();
      expect(result.insert(1, 'c')).toNotExist();
      expect(arrayInsert).toHaveBeenCalled().toHaveBeenCalledWith(1, 'c');
    });

    it('should provide remove', function () {
      var arrayRemove = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b']),
        arrayRemove: arrayRemove
      }]));
      expect(result.remove).toBeA('function');
      expect(arrayRemove).toNotHaveBeenCalled();
      expect(result.remove(2)).toNotExist();
      expect(arrayRemove).toHaveBeenCalled().toHaveBeenCalledWith(2);
    });

    it('should provide unshift', function () {
      var arrayUnshift = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b']),
        arrayUnshift: arrayUnshift
      }]));
      expect(result.unshift).toBeA('function');
      expect(arrayUnshift).toNotHaveBeenCalled();
      expect(result.unshift('c')).toNotExist();
      expect(arrayUnshift).toHaveBeenCalled().toHaveBeenCalledWith('c');
    });

    it('should provide shift', function () {
      var arrayShift = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b', 'c']),
        arrayShift: arrayShift
      }]));
      expect(result.shift).toBeA('function');
      expect(arrayShift).toNotHaveBeenCalled();
      expect(result.shift()).toBe('a');
      expect(arrayShift).toHaveBeenCalled().toHaveBeenCalledWith();
    });

    it('should provide forEach', function () {
      var callback = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b', 'c'])
      }]));
      expect(result.forEach).toBeA('function');
      expect(callback).toNotHaveBeenCalled();
      result.forEach(callback);
      expect(callback).toHaveBeenCalled();
      expect(callback.calls.length).toBe(3);
      expect(callback.calls[0].arguments).toEqual(['foo[0]', 0]);
      expect(callback.calls[1].arguments).toEqual(['foo[1]', 1]);
      expect(callback.calls[2].arguments).toEqual(['foo[2]', 2]);
    });

    it('should provide map', function () {
      var callback = (0, _expect.createSpy)(function (name) {
        return { whatever: true, name: name };
      }).andCallThrough();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        value: fromJS(['a', 'b', 'c'])
      }]));
      expect(result.map).toBeA('function');
      expect(callback).toNotHaveBeenCalled();
      var mapResult = result.map(callback);
      expect(size(mapResult), 3);
      expect(getIn(mapResult, 0)).toEqual({ whatever: true, name: 'foo[0]' });
      expect(getIn(mapResult, 1)).toEqual({ whatever: true, name: 'foo[1]' });
      expect(getIn(mapResult, 2)).toEqual({ whatever: true, name: 'foo[2]' });
      expect(callback).toHaveBeenCalled();
      expect(callback.calls.length).toBe(3);
      expect(callback.calls[0].arguments).toEqual(['foo[0]', 0]);
      expect(callback.calls[1].arguments).toEqual(['foo[1]', 1]);
      expect(callback.calls[2].arguments).toEqual(['foo[2]', 2]);
    });

    it('should provide swap', function () {
      var arraySwap = (0, _expect.createSpy)();
      var result = _createFieldArrayProps2.default.apply(undefined, defaultProps.concat([{
        arraySwap: arraySwap,
        value: fromJS(['a', 'b', 'c'])
      }]));
      expect(result.swap).toBeA('function');
      expect(arraySwap).toNotHaveBeenCalled();
      expect(result.swap(0, 2)).toNotExist();
      expect(arraySwap).toHaveBeenCalled().toHaveBeenCalledWith(0, 2);
    });
  });
};

describeCreateFieldProps('createFieldArrayProps.plain', _plain2.default, (0, _addExpectations2.default)(_expectations2.default));
describeCreateFieldProps('createFieldArrayProps.immutable', _immutable2.default, (0, _addExpectations2.default)(_expectations4.default));